<?php

namespace Drupal\tsk\Services;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\KeyValueStore\KeyValueExpirableFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Delete the specified temporary storages.
 *
 * @package Drupal\tsk
 */
class TskService {

  /**
   * An entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The storage factory creating the backend to store the data.
   *
   * @var \Drupal\Core\KeyValueStore\KeyValueExpirableFactoryInterface
   */
  protected $storageFactory;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\KeyValueStore\KeyValueExpirableFactoryInterface $storage_factory
   *   The key/value store factory.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, KeyValueExpirableFactoryInterface $storage_factory) {
    $this->entityTypeManager = $entity_type_manager;
    $this->storageFactory = $storage_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('keyvalue.expirable')
    );
  }

  /**
   * Delete all temporary storages, which are specified in a TskEntity.
   */
  public function killAll() {
    $tsk_entities = $this->entityTypeManager->getStorage('tsk_entity')->loadMultiple();
    foreach ($tsk_entities as $tsk_entity) {
      $this->kill($tsk_entity->collection, $tsk_entity->private, $tsk_entity->kill_all, $tsk_entity->key);
    }
  }

  /**
   * Delete one selected temporary storage.
   *
   * @param string $collection
   *   The collection name to use for this key/value store.
   * @param bool $private
   *   Is the tempstore private (true) or shared (false)?
   * @param bool $kill_all
   *   Kill all (true) or a single item (false)?
   * @param string $key
   *   The key of the stored data.
   */
  public function kill(string $collection, bool $private = FALSE, bool $kill_all = TRUE, string $key = '') {

    $storage = $private ? $this->storageFactory->get("tempstore.private.$collection") : $this->storageFactory->get("tempstore.shared.$collection");

    if ($kill_all) {
      $storage->deleteAll();
    } else {
      $storage->delete($key);
    }
  }

}
